;;; dos-fns.el --- MS-Dos specific functions.

;; Copyright (C) 1991, 1993 Free Software Foundation, Inc.

;; Maintainer: Morten Welinder (terra@diku.dk)
;; Keywords: internal

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;; Part of this code is taken from (or derived from) demacs.

;;; Code:

;;; <MULE-COMMENT>
;;; Mule distinguishes text and binary file type and maintains mode
;;; line with its own way, according to *-coding-system variable.
;;; (by Mineharu Takahara mtakahar@jp.oracle.com)
;;; </MULE-COMMENT>
(or (boundp 'MULE)
    (progn

;;; Add %t: into the mode line format just after the open-paren.
(let ((tail (member "   %[(" mode-line-format)))
  (setcdr tail (cons (purecopy "%t:")
		     (cdr tail))))
))

;; Use ";" instead of ":" as a path separator (from files.el).
(setq path-separator ";")

;; Set the null device (for compile.el).
(setq grep-null-device "NUL")

;; Set the grep regexp to match entries with drive letters.
(setq grep-regexp-alist
  '(("^\\(\\([a-zA-Z]:\\)?[^:( \t\n]+\\)[:( \t]+\\([0-9]+\\)[:) \t]" 1 3)))

;; patch 96.4.28  by K.Fujii
(or (boundp 'MULE)
    (progn

(defvar file-name-buffer-file-type-alist
  '(
    ("[:/].*config.sys$" . nil)		; config.sys text
    ("\\.elc$" . t)			; emacs stuff
    ("\\.\\(obj\\|exe\\|com\\|lib\\|sys\\|chk\\|out\\|bin\\|ico\\|pif\\)$" . t)
					; MS-Dos stuff
    ("\\.\\(arc\\|zip\\|pak\\|lzh\\|zoo\\)$" . t)
					; Packers
    ("\\.\\(a\\|o\\|tar\\|z\\|gz\\|taz\\)$" . t)
					; Unix stuff
    ("\\.tp[ulpw]$" . t)
					; Borland Pascal stuff
    ("[:/]tags$" . t)
					; Emacs TAGS file
    )
  "*Alist for distinguishing text files from binary files.
Each element has the form (REGEXP . TYPE), where REGEXP is matched
against the file name, and TYPE is nil for text, t for binary.")

(defun find-buffer-file-type (filename)
  (let ((alist file-name-buffer-file-type-alist)
	(found nil)
	(code nil))
    (let ((case-fold-search t))
      (setq filename (file-name-sans-versions filename))
      (while (and (not found) alist)
	(if (string-match (car (car alist)) filename)
	    (setq code (cdr (car alist))
		  found t))
	(setq alist (cdr alist))))
    (if found
	(cond((memq code '(nil t)) code)
	     ((and (symbolp code) (fboundp code))
	      (funcall code filename)))
      default-buffer-file-type)))

(defun find-file-binary (filename) 
  "Visit file FILENAME and treat it as binary."
  (interactive "FFind file binary: ")
  (let ((file-name-buffer-file-type-alist '(("" . t))))
    (find-file filename)))

(defun find-file-text (filename) 
  "Visit file FILENAME and treat it as a text file."
  (interactive "FFind file text: ")
  (let ((file-name-buffer-file-type-alist '(("" . nil))))
    (find-file filename)))

(defun find-file-not-found-set-buffer-file-type ()
  (save-excursion
    (set-buffer (current-buffer))
    (setq buffer-file-type (find-buffer-file-type (buffer-file-name))))
  nil)

;;; To set the default file type on new files.
(add-hook 'find-file-not-found-hooks 'find-file-not-found-set-buffer-file-type)
)) ; end, not MULE

(defvar msdos-shells '("command.com" "4dos.com" "ndos.com")
  "*List of shells that use `/c' instead of `-c' and a backslashed command.")

(defconst register-name-alist
  '((ax . 0) (bx . 1) (cx . 2) (dx . 3) (si . 4) (di . 5)
    (cflag . 6) (flags . 7)
    (al . (0 . 0)) (bl . (1 . 0)) (cl . (2 . 0)) (dl . (3 . 0))
    (ah . (0 . 1)) (bh . (1 . 1)) (ch . (2 . 1)) (dh . (3 . 1))))

(defun make-register ()
  (make-vector 8 0))

(defun register-value (regs name)
  (let ((where (cdr (assoc name register-name-alist))))
    (cond ((consp where)
	   (let ((tem (aref regs (car where))))
	     (if (zerop (cdr where))
		 (% tem 256)
	       (/ tem 256))))
	  ((numberp where)
	   (aref regs where))
	  (t nil))))

(defun set-register-value (regs name value)
  (and (numberp value)
       (>= value 0)
       (let ((where (cdr (assoc name register-name-alist))))
	 (cond ((consp where)
		(let ((tem (aref regs (car where)))
		      (value (logand value 255)))
		  (aset regs
			(car where)
			(if (zerop (cdr where))
			    (logior (logand tem 65280) value)
			  (logior (logand tem 255) (lsh value 8))))))
	       ((numberp where)
		(aset regs where (logand value 65535))))))
  regs)

(defsubst intdos (regs)
  (int86 33 regs))

;; Extra stub to functions in src/frame.c
;; Emacs aborts during dump if the following don't have a doc string.
(defun window-frame (window)
  "Return the frame that WINDOW resides on."
  (selected-frame))
(defun raise-frame (frame)
  "Raise FRAME to the top of the desktop."
  nil)
(defun select-frame (frame &optional no-enter)
  "Select FRAME for input events."
  (selected-frame))

;; patch 94.9.30  by K.Fujii
;; font-load-pc98 on PC-98*1(normal and internal-terminal)
(if (fboundp 'si:font-load-pc98)
    (progn

      (defvar inhibit-install-font-pc98 t)
      (defvar font-directory-pc98 nil)
      (defvar font-bold-directory-pc98 nil)
      (defvar font-italic-directory-pc98 nil)
      (defvar font-bold-file-pc98 nil)
      (defvar font-italic-file-pc98 nil)
      (defvar init-font-pc98-done nil)

      (defconst font-env-bold "EMACSBOLD")
      (defconst font-env-italic "EMACSITALIC")
      (defconst font-file-bold-default "bold.fot")
      (defconst font-file-italic-default "italic.fot")


      (defun check-machine-mode-and-internal-terminal (prompt)
	(if (not (eq (dos-machine-mode t) 'pc98))
	    (error (concat prompt "This machine is not pc98.")))
	(if (not (string= (getenv "TERM") "internal98"))
	    (error (concat prompt "Now running not internal-terminal or pc-window."))))

      (defun setup-font-dir-pc98 ()
	(let (var)
	  (if (not (stringp (setq var font-directory-pc98)))
	      (if (not (stringp (setq var (getenv "HOME"))))
		  (setq var default-directory)))
	  var))

      (defun env-to-file-name (env func)
	(let (name)
	  (setq name (getenv env))
	  (if (stringp name)
	      (setq name (funcall func name)))
	  name))

      (defun setup-font-dir-bold-pc98 ()
	(let (var)
	  (if (not (stringp (setq var font-bold-directory-pc98)))
	      (if (not (stringp (setq var (env-to-file-name
					   font-env-bold 'file-name-directory))))
		  (setq var (setup-font-dir-pc98))))
	  (file-name-as-directory var)))

      (defun setup-font-dir-italic-pc98 ()
	(let (var)
	  (setq var font-italic-directory-pc98)
	  (if (not (stringp (setq var font-italic-directory-pc98)))
	      (if (not (stringp (setq var (env-to-file-name
					   font-env-italic 'file-name-directory))))
		  (setq var (setup-font-dir-pc98))))
	  (file-name-as-directory var)))

      (defun setup-font-file-bold-pc98 ()
	(let (var)
	  (if (not (stringp (setq var font-bold-file-pc98)))
	      (if (not (stringp (setq var (env-to-file-name
					   font-env-bold 'file-name-nondirectory))))
		  (setq var font-file-bold-default)))
	  var))

      (defun setup-font-file-italic-pc98 ()
	(let (var)
	  (if (not (stringp (setq var font-italic-file-pc98)))
	      (if (not (stringp (setq var (env-to-file-name
					   font-env-italic
					   'file-name-nondirectory))))
		  (setq var font-file-italic-default)))
	  var))

      (defun font-load-pc98 (&optional font dir file fast)
	(interactive)
	(if (null inhibit-install-font-pc98)
	    (progn
	      (check-machine-mode-and-internal-terminal "font-load-pc98: ")
	      (let (font-dir font-file font-path)
		(if (and (interactive-p)
			 (null font))
		    (setq font (read-string "font-load-pc98: Font-name [Bold/Italic] ? "
					    nil)))
		(cond ((and (stringp font)
			    (or (= (string-to-char font) ?b)
				(= (string-to-char font) ?B)))
		       (setq font 'pc98-font-bold))
		      ((and (stringp font)
			    (or (= (string-to-char font) ?i)
				(= (string-to-char font) ?I)))
		       (setq font 'pc98-font-italic))
		      (t
		       (error "font-load-pc98: Unknown font!!")))
		(if (not (stringp dir))
		    (if (eq font 'pc98-font-bold)
			(setq font-dir (setup-font-dir-bold-pc98))
		      (setq font-dir (setup-font-dir-italic-pc98)))
		  (setq font-dir dir))
		(if (not (stringp file))
		    (progn
		      (if (eq font 'pc98-font-bold)
			  (setq font-file (setup-font-file-bold-pc98))
			(setq font-file (setup-font-file-italic-pc98))))
		  (setq font-file file))
		(setq font-path (concat font-dir font-file))
		(if (and (interactive-p)
			 (not (stringp file)))
		    (setq font-path (read-file-name
				     (format "font-load-pc98: (default %s) ? "
					     font-file)
				     font-dir font-path nil nil)))
		(setq font-path (expand-file-name font-path nil))
		(if (file-exists-p font-path)
		    (progn
		      (if (null fast)
			  (message "font-load-pc98: Loading font..."))
		      (si:font-load-pc98 font font-path fast)
		      (if (null fast)
			  (message "font-load-pc98: Loading font...done.")))
		  (error "font-load-pc98: font-file %s is not defined."
			 font-file))
		)))
	nil)

      (defun init-font-pc98 (&optional bold italic)
	(if (and (null inhibit-install-font-pc98)
		 (null init-font-pc98-done))
	    (progn
	      ;; chech machine mode and internal-terminal mode.
	      (check-machine-mode-and-internal-terminal "init-font-pc98")
	      ;; load bold font.
	      (si:font-load-pc98 'pc98-font-bold
				 (if (null bold)
				     (concat (setup-font-dir-bold-pc98)
					     (setup-font-file-bold-pc98))
				   bold)
				 t)

	      ;; load italic font.
	      (si:font-load-pc98 'pc98-font-italic
				 (if (null italic)
				     (concat (setup-font-dir-italic-pc98)
					     (setup-font-file-italic-pc98))
				   italic)
				 t)

	      (setq init-font-pc98-done t)
	      ))
	nil)
      ))

(autoload 'sshell "sshel" "" t nil)
;; end of patch
