;; fepctrl.el: Kana-Kanji Translation FEP Control Library.
;;
;; Edition History:
;; 1.1 1991/11/28 Manabu Higashida Creation.
;; 1.2 1993/02/25 Manabu Higashida Modify for FEPCTRL 1.5
 
;; Copyright (C) 1991 Free Software Foundation, Inc.

;; This file is part of GNU Emacs.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY.  No author or distributor
;; accepts responsibility to anyone for the consequences of using it
;; or for whether it serves any particular purpose or works at all,
;; unless he says so in writing.  Refer to the GNU Emacs General Public
;; License for full details.

;; Everyone is granted permission to copy, modify and redistribute
;; GNU Emacs, but only under the conditions described in the
;; GNU Emacs General Public License.   A copy of this license is
;; supposed to have been given to you along with GNU Emacs so you
;; can know your rights and responsibilities.  It should be in a
;; file named COPYING.  Among other things, the copyright notice
;; and this notice must be preserved on all copies.

(defconst fep-name-list
  '("(none)"	;  0
    "PC98A"	;  1
    "PC98B"	;  2
    "PC98C"	;  3
    "MSKANJI"	;  4
    "VJE"	;  5
    "ATOK6"	;  6
    "ATOK7"	;  7
    "MTTK86"	;  8
    "MTTK2"	;  9
    "KATANA"	; 10
    "FIXER"	; 11
    "EGB2"	; 12
    "EGB3"	; 13
    "WXP"	; 14
    "WX2"	; 15
    "MGR2"	; 16
    "JJ"	; 17
    "NEC"	; 18
    "DFJ"	; 19
    "DANGO"	; 20
    "OTEMOTO"	; 21
    "OMAC"	; 22
    "AJIP1"	; 23
    "JOKER3"	; 24
    "KAZE"	; 25
    "OAK"	; 26
    "MKK"	; 27
    "B16"	; 28
    "RICOH"	; 29
    "WXPJ"	; 30
    "MIJ"	; 31
    "FEPEX"	; 32
    "AT6AX"	; 33
    "AT6IBM"	; 34
    "AT7IBM"	; 35
    "AT7DOSV"	; 36
    "IBMIAS"	; 37
    ))

(defvar fep-name nil)

(defvar fep-stack nil)
(defun fep-push ()
  (let ((flag (fep-get-mode)))
    (setq fep-stack (cons flag fep-stack))
    (and flag
	 (fep-force-off))))

(defun fep-pop ()
  (and (car-safe fep-stack)
       (fep-force-on))
  (setq fep-stack (cdr-safe fep-stack)))

  
(or (fboundp 'si:fep-init)
    (fset 'si:fep-init (symbol-function 'fep-init)))

(defun fep-init ()
  ""
  (setq fep-stack nil)
  (let ((fep-id (si:fep-init)))
    (if (or (not (numberp fep-id))
	    (zerop fep-id))
	(setq inhibit-fep-control t)
      (setq fep-name (nth fep-id fep-name-list)))))

(defun fep-sync ()
  (interactive)
  (setq fep-stack nil)
  (if fep-mode
      (fep-force-on)
    (fep-force-off)))

;(or (fboundp 'si:fep-get-mode)
;    (fset 'si:fep-get-mode (symbol-function 'fep-get-mode)))
;
;(defun fep-get-mode ()
;  (not (= (si:fep-get-mode) 0)))

(defun toggle-fep-mode ()
  (interactive)
  (let ((flag (fep-get-mode)))
    (if flag
	(fep-force-off)
      (fep-force-on))
    (setq fep-mode flag)))

;;;

(or (fboundp 'si:universal-argument)
    (fset 'si:universal-argument (symbol-function 'universal-argument)))

(defun universal-argument ()
  ""
  (interactive nil)
  (fep-push)
  (unwind-protect
      (si:universal-argument)
    (fep-pop)))

;;;

(or (fboundp 'si:quoted-insert)
    (fset 'si:quoted-insert (symbol-function 'quoted-insert)))

(defun quoted-insert (arg)
  ""
  (interactive "*p")
  (fep-push)
  (unwind-protect
      (si:quoted-insert arg)
    (fep-pop)))

;;;

(defvar dos-isearch-fep-mode nil)
(defvar dos-isearch-first-toggle-fep-mode nil)

(add-hook 'isearch-mode-hook
	  (function
	   (lambda ()
	     (setq dos-isearch-fep-mode (fep-get-mode))
	     (and (null dos-isearch-fep-mode)
		  (not (eq (dos-machine-mode t) 'go32win))
		  (null dos-isearch-first-toggle-fep-mode)
		  (setq dos-isearch-first-toggle-fep-mode t)
		  (fep-force-on))
	     (fep-force-off)
	     )))

(add-hook 'isearch-mode-end-hook
	  (function
	   (lambda ()
	     (and dos-isearch-fep-mode
		  (fep-force-on))
	     )))

(defvar dos-isearch-fep-read-string nil)

(add-hook 'minibuffer-setup-hook
	  (function
	   (lambda ()
	     (if dos-isearch-fep-read-string
		 (fep-force-on)))))

(add-hook 'minibuffer-exit-hook
	  (function
	   (lambda ()
	     (and dos-isearch-fep-read-string
		  (fep-force-off))
	     (setq dos-isearch-fep-read-string nil))))

(or (fboundp 'si:isearch-fep-read-string)
    (fset 'si:isearch-fep-read-string
	  (symbol-function 'isearch-fep-read-string)))

(defun isearch-fep-read-string ()
  (setq dos-isearch-fep-read-string t)
  (unwind-protect
      (si:isearch-fep-read-string)
    (setq dos-isearch-fep-read-string nil)
    (fep-force-off)))

;(or (fboundp 'si:read-string)
;    (fset 'si:read-string (symbol-function 'read-string)))
;
;(defun read-string (args &optional arg1)
;  (fep-on)
;  (unwind-protect
;      (funcall 'si:read-string args arg1)
;    (fep-off)))

;;;

;
; This doesn't work correctly to handle C-h.
;
;(or (fboundp 'si:help-for-help)
;    (fset 'si:help-for-help (symbol-function 'help-for-help)))
;
;(defun help-for-help ()
;  (interactive)
;  (fep-off)
;  (unwind-protect
;      (si:help-for-help)
;    (fep-on)))

