;; internal.el -- setup support for PC keyboards and screens, internal terminal

;; Copyright (C) 1993, 1994 Free Software Foundation, Inc.

;; Author: Morten Welinder <terra@diku.dk>
;; Version: 1,02

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
;; ---------------------------------------------------------------------------
;; ---------------------------------------------------------------------------
;; keyboard setup -- that's simple!
;; patch 94.10.8 for Mule-2.3@19.30  by K.Fujii

(defvar init-terminal-mode-after-hook nil)

(set-input-mode nil nil 0)

(defvar user-function-key
  '(([f1]    . find-file)		;f.1
    ([f2]    . other-window)		;f.2
    ([f3]    . undo)			;f.3
    ([f4]    . tabify)			;f.4
    ([f5]    . dired)			;f.5
    ([f6]    . isearch-forward-regexp)	;f.6
    ([f7]    . isearch-forward)		;f.7
    ([f8]    . query-replace)		;f.8
    ([f9]    . query-replace-regexp)	;f.9
    ([f10]   . sshell)			;f.10
    ([S-f1]  . find-file-other-window)	;Shift + f.1
    ([S-f2]  . split-window-vertically)	;Shift + f.2
    ([S-f3]  . split-window-horizontally);Shift + f.3
    ([S-f4]  . untabify)		;Shift + f.4
    ([S-f5]  . buffer-menu)		;Shift + f.5
    ([S-f6]  . isearch-backward-regexp)	;Shift + f.6
    ([S-f7]  . isearch-backward)	;Shift + f.7
    ([S-f8]  . query-replace)		;Shift + f.8
    ([S-f9]  . set-mark-command)	;Shift + f.9
    ([S-f10] . mark-defun)		;Shift + f.10
    ([S-left] . backward-word)		;Shift + left
    ([C-left] . beginning-of-line)	;Ctrl + left
    ([S-right] . forward-word)		;Shift + right
    ([C-right] . end-of-line)		;Ctrl + right
    ([S-home] . end-of-buffer)		;Shift + HOME
    ))

(defun define-user-function-keys ()
  (let ((keys user-function-key))
    (while keys
      (define-key global-map
	(car (car keys))
	(cdr (car keys))
	t)
      (setq keys (cdr keys)))))

(if (eq window-system 'pc)
    (add-hook 'after-user-init-hook 'define-user-function-keys)
  (define-user-function-keys))
;; end of patch

(define-key function-key-map [backspace] "\177") ; Normal behaviour for BS
(define-key function-key-map [delete] "\C-d")    ; ... and Delete
(define-key function-key-map [tab] [?\t])
(define-key function-key-map [linefeed] [?\n])
(define-key function-key-map [clear] [11])
(define-key function-key-map [return] [13])
(define-key function-key-map [escape] [?\e])
(define-key function-key-map [M-backspace] [?\M-\d])
(define-key function-key-map [M-delete] [?\M-\d])
(define-key function-key-map [M-tab] [?\M-\t])
(define-key function-key-map [M-linefeed] [?\M-\n])
(define-key function-key-map [M-clear] [?\M-\013])
(define-key function-key-map [M-return] [?\M-\015])
(define-key function-key-map [M-escape] [?\M-\e])
(put 'backspace 'ascii-character 127)
(put 'delete 'ascii-character 127)
(put 'tab 'ascii-character ?\t)
(put 'linefeed 'ascii-character ?\n)
(put 'clear 'ascii-character 12)
(put 'return 'ascii-character 13)
(put 'escape 'ascii-character ?\e)
;; ---------------------------------------------------------------------------
;; We want to do this when Emacs is started because it depends on the
;; country code.
;; This code is modified for Mule-2.0  by K.Fujii
(and (eq (dos-machine-mode t) 'ibmpc-us)
     (progn
       (setq-default mc-flag nil)
       (setq-default default-mc-flag nil)
       (require 'disp-table)
       (standard-display-8bit 127 254)
       (let* ((i 128)
	      (modify (function
		       (lambda (ch sy) 
			 (modify-syntax-entry ch sy text-mode-syntax-table)
			 (if (boundp 'tex-mode-syntax-table)
			     (modify-syntax-entry ch sy tex-mode-syntax-table))
			 (modify-syntax-entry ch sy (standard-syntax-table))
			 )))
	      (table (standard-case-table))
	      ;; The following are strings of letters, first lower then upper case.
	      ;; This will look funny on terminals which display other code pages.
	      (chars
	       (cond
		((= dos-codepage 850)
		 "\207\200\201\232\202\220\204\216\206\307\240\265\211\323\241\326\221\222\223\342\224\231\340\233\325\351\227\353\230Y\354\355\241I\243\351\244\245\320\321\347\350")
		((= dos-codepage 865)
		 "\207\200\201\232\202\220\203A\204\216\205A\206\217\210E\211E\212E\213I\214I\215I\221\222\223O\224\231\225O\226U\243U\230Y\233\235\240A\241I\242O\243U\244\245")
		;; default is 437
		(t "\207\200\201\232\202\220\203A\204\216\205A\206\217\210E\211E\212E\212I\214I\215I\221\222\223O\224\231\225O\226U\243U\230Y\240A\241I\242O\243U\244\245"))))

	 (while (< i 256)
	   (funcall modify i "_")
	   (setq i (1+ i)))

	 (setq i 0)
	 (while (< i (length chars))
	   (let ((ch1 (aref chars i))
		 (ch2 (aref chars (1+ i))))
	     (if (> ch2 127)
		 (set-case-syntax-pair ch2 ch1 table))
	     (setq i (+ i 2))))
	 (save-excursion
	   (mapcar (lambda (b) (set-buffer b) (set-case-table table))
		   (buffer-list)))
	 (set-standard-case-table table))
       ))

(defvar default-faces-alist
  ;;(face		fore	back	    (bold italic underline))
  '((bold		nil	nil	    (t    nil    nil))
    (bold-italic	nil	nil	    (t    nil    t))
;   (default		"white"	nil	    (nil  nil    nil))
    (highlight		"black"	"green"	    (nil  t      nil))
    (italic		nil	nil	    (nil  t      nil))
    (modeline		"black"	"lightcyan" (nil  t      nil))
    (region		nil	"yellow"    (nil  nil    nil))
    (secondary-selection nil	"magenta"   (nil  nil    nil))
    (underline		nil	nil	    (nil  nil    t))))

(defvar default-attribute
  (if (eq (dos-machine-mode t) 'pc98)
      '("white" . "black")
    '("black" . "white")))

(defun set-default-color (fore back)
  (set-face-foreground 'default fore)
  (set-face-background 'default back)
  (modify-frame-parameters (selected-frame)
			   (list (cons 'foreground-color fore)
				 (cons 'background-color back)))
  (sit-for 0)
  (redraw-display))

(defvar msdos-menu-faces-alist
  (if (eq (dos-machine-mode t) 'pc98)
      '(("blue" . nil)
	("blue" . nil)
	(select . "green"))
    '(("blue" . "white")
      ("blue" . "lightgray")
      (select . "red"))))
    
;; patch 94.10.4, 95.12.26  by K.Fujii
;; for init-font-pc98
(if (eq (dos-machine-mode t) 'pc98)
    (progn
      (defun pc98-after-init ()
	(if (and (fboundp 'init-font-pc98)
		 (eq (dos-machine-mode t) 'pc98))
	    (progn
	      (setq inhibit-install-font-pc98 nil)
	      (init-font-pc98))
	  nil))
      (if (null window-system)
	  (pc98-after-init)
	(add-hook 'after-user-init-hook 'pc98-after-init))))
;; end of patch

(if (null window-system)
    (progn
      (defvar default-frame-alist nil)
      (modify-frame-parameters (selected-frame)
			       '((font . "default")
				 (background-color . "black")
				 (foreground-color . "white")))

      (and default-attribute
	   (consp default-attribute)
	   (set-default-color (car default-attribute)
			      (cdr default-attribute)))

      (and default-faces-alist
	   (consp default-faces-alist)
	   (let ((faces default-faces-alist))
	     (while faces
		 (let* ((face (car faces))
			(f (nth 0 face))
			(fore (nth 1 face))
			(back (nth 2 face))
			(biu (nth 3 face)))
		   (set-face-foreground f fore)
		   (set-face-background f back)
		   (let* ((bold (nth 0 biu))
			  (italic (nth 1 biu))
			  (underline (nth 2 biu)))
		     (set-face-bold-p f bold)
		     (set-face-italic-p f italic)
		     (set-face-underline-p f underline))
		   (setq faces (cdr faces))))))
      (make-face 'msdos-menu-active-face)
      (make-face 'msdos-menu-passive-face)
      (make-face 'msdos-menu-select-face)
      (set-face-foreground 'msdos-menu-active-face
			   (cdr (nth 0 msdos-menu-faces-alist)))
      (set-face-background 'msdos-menu-active-face
			   (car (nth 0 msdos-menu-faces-alist)))
      (set-face-foreground 'msdos-menu-passive-face
			   (cdr (nth 1 msdos-menu-faces-alist)))
      (set-face-background 'msdos-menu-passive-face
			   (car (nth 1 msdos-menu-faces-alist)))
      (set-face-background 'msdos-menu-select-face
			   (cdr (nth 2 msdos-menu-faces-alist)))
      (and init-terminal-mode-after-hook
	   (run-hooks 'init-terminal-mode-after-hook))
      ))

;;
;; mouse
;; ported Go32win to DOS by Y.Matsushima
;;
(if (boundp 'MULE)
	(progn
	  (load-library "mldrag")
	  (defun mouse-select-window (click)
		"(Ov.go32)Select the window clicked on; don't move point."
		(interactive "e")
		(mouse-minibuffer-check click)
		(let ((oframe (selected-frame))
			  (frame (window-frame (posn-window (event-start click)))))
		  (select-window (posn-window (event-start click)))
;;		  (raise-frame frame)
;;		  (select-frame frame)
;;		  (or (eq frame oframe)
;;			  (set-mouse-position (selected-frame) (1- (frame-width)) 0))
;;		  (unfocus-frame)
		  ))

	  (defvar secondary_yank_buffer nil
	    "mouse 'SECONDARY region. Clipboard emulation string buffer.
(for Go32win DOS Patch)")

	  (defun go32win-set-selection (type text)
		"Windows 3.1 clipboard put. for mouse secondary region."
		(if (and (eq type 'SECONDARY)	; mouse secondary region/yank Onry.
				 (stringp text))
			(setq secondary_yank_buffer text)))

	  (defun go32win-get-selection (type)
		"Windows 3.1 clipboard get. for mouse secondary yank."
		(if (eq type 'SECONDARY)
			(if (featurep 'mule)
			(code-convert-string (or secondary_yank_buffer "")  *internal* *ctext*)
			(or secondary_yank_buffer ""))))
		
	  (if (eq kill-ring nil)
		  (kill-new ""))
		   
	  (defalias 'x-set-selection 'go32win-set-selection)
	  (defalias 'x-get-selection 'go32win-get-selection)

	  (global-unset-key [M-mouse-1])
	  (global-unset-key [M-drag-mouse-1])
	  (global-unset-key [M-down-mouse-1])
	  (global-unset-key [M-mouse-3])
	  (global-unset-key [M-mouse-2])
	  (global-unset-key [C-down-mouse-3])
	  (global-unset-key [mouse-3])
	  (global-unset-key [mode-line down-mouse-1])
	  (global-unset-key [mode-line mouse-3])

	  (global-set-key [C-mouse-1] 'mouse-start-secondary)
	  (global-set-key [C-drag-mouse-1] 'mouse-set-secondary)
	  (global-set-key [C-down-mouse-1] 'mouse-drag-secondary)
	  (global-set-key [C-mouse-2] 'mouse-secondary-save-then-kill)
	  (global-set-key [S-C-mouse-2] 'mouse-yank-secondary)

	  (global-set-key [mouse-2]	'mouse-yank-at-click)
	  (global-set-key [S-mouse-2] 'mouse-save-then-kill)

	  (global-set-key [mode-line down-mouse-1] 'mldrag-drag-mode-line)
	  (global-set-key [vertical-line down-mouse-1] 'mldrag-drag-vertical-line)
	  (global-set-key [mode-line S-mouse-2] 'mouse-delete-window)
	  (global-set-key [mode-line S-mouse-1] 'buffer-menu)))

